// Abstract class to be subclassed when creating Tool objects
// When subclassing, note that Multiscroll contracts to set startX, Y, scrollX, Y,
// client and settings prior to calling.

Multiscroll.Tool = function(methods) {
	if (methods) {
		this.start = methods.start || function() { return null; };
		this.stop = methods.stop || function() { return null; };
		this.execute = methods.execute || function() { return null; };
		this.loop = methods.loop || function() { return null; };
		this.insertMarker = methods.insertMarker || function() { return null; };
		this.removeMarker = methods.removeMarker || function() { return null; };
	}

	this.active = false;
	this.done = false;
	
	this.markerNode = null;

	this.startX = this.startY = 0;
	this.scrollX = this.scrollY = 0;
	this.client = {
		frame : null,
		document : null,
		body : null,
		window : null
	};
	this.settings = null;
};

Multiscroll.Tool.None = function(parent) {
	this.parent = parent;
}
Multiscroll.Tool.None.prototype = new Multiscroll.Tool({
	start : function(evt) {
		for (action in this.parent.action)
			if (this.parent.action[action].active)
				this.parent.action[action].tool.stop(evt);
		return true;
	}
});
Multiscroll.Tool.None.prototype.active getter= function() { 
	return false;
};
Multiscroll.Tool.None.prototype.active setter= function(value) {
	if (value) {
		for (action in this.parent.action)
			if (this.parent.action[action].active)
				this.parent.action[action].active = false;
	} else if (this.active) {
		this.parent.action.initial.active = true;
	}
	return value;
};
Multiscroll.Tool.None.prototype.done getter= function() { 
	return false;
};


// mergeObjects(base, [overlay, ... ]) - does what it says: overlays overlay, ... onto  
// base and returns the resulting object
// Note that this will DESTROY getters in overlay unless protected by an object
// wrapper not in base at that point in the process.
// Instead of mergeObjects(base, { get attr() { ... } }); use base.attr getter= { ... };
function mergeObjects(base) {
	if (!(base instanceof Object))
		return base;
	for (var i = 1; i < arguments.length; i++)
		if (arguments[i] instanceof Object)
			for (var j in arguments[i]) {
				if (base[j] == undefined || base[j] == null)
					base[j] = arguments[i][j];
				else if (base[j] instanceof Object && arguments[i][j] instanceof Object)
					mergeObjects(base[j], arguments[i][j]);
			}
	return base;
}